@@ -1,5 +1,5 @@
 //
-// Copyright 2020 Staysail Systems, Inc. <info@staysail.tech>
+// Copyright 2021 Staysail Systems, Inc. <info@staysail.tech>
 // Copyright 2018 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
@@ -15,8 +15,9 @@ struct nni_aio_expire_q {
 	nni_mtx  eq_mtx;
 	nni_cv   eq_cv;
 	nni_list eq_list;
-	nni_aio *eq_aio; // currently expiring (task dispatch)
+	uint32_t eq_len;
 	nni_thr  eq_thr;
+	nni_time eq_next; // next expiration
 	bool     eq_exit;
 };
 
@@ -98,6 +99,7 @@ static nni_reap_list aio_reap_list = {
 };
 
 static void nni_aio_expire_add(nni_aio *);
+static void nni_aio_expire_rm(nni_aio *);
 
 void
 nni_aio_init(nni_aio *aio, nni_cb cb, void *arg)
@@ -114,7 +116,7 @@ void
 nni_aio_fini(nni_aio *aio)
 {
 	nni_aio_cancel_fn fn;
-	void *            arg;
+	void	     *arg;
 	nni_aio_expire_q *eq = aio->a_expire_q;
 
 	// This is like aio_close, but we don't want to dispatch
@@ -123,10 +125,10 @@ nni_aio_fini(nni_aio *aio)
 	// We also wait if the aio is being expired.
 	nni_mtx_lock(&eq->eq_mtx);
 	aio->a_stop = true;
-	nni_list_node_remove(&aio->a_expire_node);
-	while (eq->eq_aio == aio) {
+	while (aio->a_expiring) {
 		nni_cv_wait(&eq->eq_cv);
 	}
+	nni_aio_expire_rm(aio);
 	fn                = aio->a_cancel_fn;
 	arg               = aio->a_cancel_arg;
 	aio->a_cancel_fn  = NULL;
@@ -200,11 +202,11 @@ nni_aio_stop(nni_aio *aio)
 {
 	if (aio != NULL) {
 		nni_aio_cancel_fn fn;
-		void *            arg;
+		void             *arg;
 		nni_aio_expire_q *eq = aio->a_expire_q;
 
 		nni_mtx_lock(&eq->eq_mtx);
-		nni_list_node_remove(&aio->a_expire_node);
+		nni_aio_expire_rm(aio);
 		fn                = aio->a_cancel_fn;
 		arg               = aio->a_cancel_arg;
 		aio->a_cancel_fn  = NULL;
@@ -225,11 +227,11 @@ nni_aio_close(nni_aio *aio)
 {
 	if (aio != NULL) {
 		nni_aio_cancel_fn fn;
-		void *            arg;
+		void             *arg;
 		nni_aio_expire_q *eq = aio->a_expire_q;
 
 		nni_mtx_lock(&eq->eq_mtx);
-		nni_list_node_remove(&aio->a_expire_node);
+		nni_aio_expire_rm(aio);
 		fn                = aio->a_cancel_fn;
 		arg               = aio->a_cancel_arg;
 		aio->a_cancel_fn  = NULL;
@@ -350,7 +352,6 @@ nni_aio_begin(nni_aio *aio)
 		aio->a_expire_ok = false;
 		nni_mtx_unlock(&eq->eq_mtx);
 
-		nni_task_dispatch(&aio->a_task);
 		return (NNG_ECANCELED);
 	}
 	nni_task_prep(&aio->a_task);
@@ -405,11 +406,11 @@ void
 nni_aio_abort(nni_aio *aio, int rv)
 {
 	nni_aio_cancel_fn fn;
-	void *            arg;
+	void	     *arg;
 	nni_aio_expire_q *eq = aio->a_expire_q;
 
 	nni_mtx_lock(&eq->eq_mtx);
-	nni_list_node_remove(&aio->a_expire_node);
+	nni_aio_expire_rm(aio);
 	fn                = aio->a_cancel_fn;
 	arg               = aio->a_cancel_arg;
 	aio->a_cancel_fn  = NULL;
@@ -432,7 +433,7 @@ nni_aio_finish_impl(
 
 	nni_mtx_lock(&eq->eq_mtx);
 
-	nni_list_node_remove(&aio->a_expire_node);
+	nni_aio_expire_rm(aio);
 	aio->a_result     = rv;
 	aio->a_count      = count;
 	aio->a_cancel_fn  = NULL;
@@ -505,92 +506,115 @@ nni_aio_list_active(nni_aio *aio)
 static void
 nni_aio_expire_add(nni_aio *aio)
 {
-	nni_list *list = &aio->a_expire_q->eq_list;
-	nni_aio * prev;
+	nni_aio_expire_q *eq = aio->a_expire_q;
 
-	// This is a reverse walk of the list.  We're more likely to find
-	// a match at the end of the list.
-	for (prev = nni_list_last(list); prev != NULL;
-	     prev = nni_list_prev(list, prev)) {
-		if (aio->a_expire >= prev->a_expire) {
-			nni_list_insert_after(list, aio, prev);
-			break;
-		}
-	}
-	if (prev == NULL) {
-		// This has the shortest time, so insert at the start.
-		nni_list_prepend(list, aio);
-		// And, as we are the latest, kick the thing.
-		nni_cv_wake(&aio->a_expire_q->eq_cv);
+	nni_list_append(&eq->eq_list, aio);
+
+	if (eq->eq_next > aio->a_expire) {
+		eq->eq_next = aio->a_expire;
+		nni_cv_wake(&eq->eq_cv);
 	}
 }
 
+static void
+nni_aio_expire_rm(nni_aio *aio)
+{
+	nni_list_node_remove(&aio->a_expire_node);
+
+	// If this item is the one that is going to wake the loop,
+	// don't worry about it.  It will wake up normally, or when we
+	// add a new aio to it. Worst case is just one spurious wake up,
+	// which we'd need to do anyway.
+}
+
 static void
 nni_aio_expire_loop(void *arg)
 {
-	nni_aio_expire_q *q    = arg;
-	nni_list *        list = &q->eq_list;
-	nni_mtx *         mtx  = &q->eq_mtx;
-	nni_cv *          cv   = &q->eq_cv;
+	nni_aio_expire_q *q   = arg;
+	nni_mtx          *mtx = &q->eq_mtx;
+	nni_cv           *cv  = &q->eq_cv;
 	nni_time          now;
+	uint32_t          exp_idx;
+	nni_aio          *expires[NNI_EXPIRE_BATCH];
 
 	nni_thr_set_name(NULL, "nng:aio:expire");
 
-	now = nni_clock();
 	nni_mtx_lock(mtx);
 
 	for (;;) {
 		nni_aio *aio;
 		int      rv;
+		nni_time next;
 
-		if ((aio = nni_list_first(list)) == NULL) {
+		next       = q->eq_next;
+		now        = nni_clock();
 
-			if (q->eq_exit) {
-				nni_mtx_unlock(mtx);
-				return;
+		// Each time we wake up, we scan the entire list of elements.
+		// We scan forward, moving up to NNI_EXPIRE_Q_SIZE elements
+		// (a batch) to a saved array of things we are going to cancel.
+		// This mostly runs in O(n), provided you don't have many
+		// elements (> NNI_EXPIRE_Q_SIZE) all expiring simultaneously.
+		aio = nni_list_first(&q->eq_list);
+		if ((aio == NULL) && (q->eq_exit)) {
+			nni_mtx_unlock(mtx);
+			return;
+		}
+		if (now < next) {
+			// Early wake up (just to reschedule), no need to
+			// rescan the list.  This is an optimization.
+			nni_cv_until(cv, next);
+			continue;
+		}
+		q->eq_next = NNI_TIME_NEVER;
+		exp_idx = 0;
+		while (aio != NULL) {
+			if ((aio->a_expire < now) &&
+			    (exp_idx < NNI_EXPIRE_BATCH)) {
+				nni_aio *nxt;
+
+				// This one is expiring.
+				expires[exp_idx++] = aio;
+				// save the next node
+				nxt = nni_list_next(&q->eq_list, aio);
+				nni_list_remove(&q->eq_list, aio);
+				// Place a temporary hold on the aio.
+				// This prevents it from being destroyed.
+				aio->a_expiring = true;
+				aio             = nxt;
+				continue;
 			}
-
-			nni_cv_wait(cv);
-
-			now = nni_clock();
-			continue;
+			if (aio->a_expire < q->eq_next) {
+				q->eq_next = aio->a_expire;
+			}
+			aio = nni_list_next(&q->eq_list, aio);
 		}
 
-		if (now < aio->a_expire) {
-			// Unexpired; the list is ordered, so we just wait.
-			nni_cv_until(cv, aio->a_expire);
-			now = nni_clock();
-			continue;
+		for (uint32_t i = 0; i < exp_idx; i++) {
+			aio = expires[i];
+			rv = aio->a_expire_ok ? 0 : NNG_ETIMEDOUT;
+
+			nni_aio_cancel_fn cancel_fn  = aio->a_cancel_fn;
+			void             *cancel_arg = aio->a_cancel_arg;
+
+			aio->a_cancel_fn  = NULL;
+			aio->a_cancel_arg = NULL;
+
+			// We let the cancel function handle the completion.
+			// If there is no cancellation function, then we cannot
+			// terminate the aio - we've tried, but it has to run
+			// to its natural conclusion.
+			if (cancel_fn != NULL) {
+				nni_mtx_unlock(mtx);
+				cancel_fn(aio, cancel_arg, rv);
+				nni_mtx_lock(mtx);
+			}
+			aio->a_expiring = false;
 		}
-
-		// The time has come for this aio.  Expire it, canceling any
-		// outstanding I/O.
-		nni_list_remove(list, aio);
-		rv = aio->a_expire_ok ? 0 : NNG_ETIMEDOUT;
-
-		nni_aio_cancel_fn cancel_fn  = aio->a_cancel_fn;
-		void *            cancel_arg = aio->a_cancel_arg;
-
-		aio->a_cancel_fn  = NULL;
-		aio->a_cancel_arg = NULL;
-		// Place a temporary hold on the aio.  This prevents it
-		// from being destroyed.
-		q->eq_aio = aio;
-
-		// We let the cancel function handle the completion.
-		// If there is no cancellation function, then we cannot
-		// terminate the aio - we've tried, but it has to run
-		// to it's natural conclusion.
-		nni_mtx_unlock(mtx);
-		cancel_fn(aio, cancel_arg, rv);
-
-		// Get updated time before reacquiring lock.
-		now = nni_clock();
-
-		nni_mtx_lock(mtx);
-
-		q->eq_aio = NULL;
 		nni_cv_wake(cv);
+
+		if (now < q->eq_next) {
+			nni_cv_until(cv, q->eq_next);
+		}
 	}
 }
 
@@ -672,7 +696,7 @@ nni_sleep_cancel(nng_aio *aio, void *arg, int rv)
 	}
 
 	aio->a_sleep = false;
-	nni_list_node_remove(&aio->a_expire_node);
+	nni_aio_expire_rm(aio);
 	nni_mtx_unlock(&eq->eq_mtx);
 
 	nni_aio_finish_error(aio, rv);
@@ -737,6 +761,7 @@ nni_aio_expire_q_alloc(void)
 	nni_mtx_init(&eq->eq_mtx);
 	nni_cv_init(&eq->eq_cv, &eq->eq_mtx);
 	NNI_LIST_INIT(&eq->eq_list, nni_aio, a_expire_node);
+	eq->eq_next = NNI_TIME_NEVER;
 	eq->eq_exit = false;
 
 	if (nni_thr_init(&eq->eq_thr, nni_aio_expire_loop, eq) != 0) {
@@ -775,7 +800,6 @@ nni_aio_sys_init(void)
 		num_thr = 256;
 	}
 
-
 	nni_aio_expire_q_list =
 	    nni_zalloc(sizeof(nni_aio_expire_q *) * num_thr);
 	nni_aio_expire_q_cnt = num_thr;
