@@ -34,13 +34,13 @@ nni_taskq_thread(void *self)
 	nni_taskq *    tq  = thr->tqt_tq;
 	nni_task *     task;
 
-	nni_mtx_lock(&tq->tq_mtx);
+        nni_thr_set_name(NULL, "nng:task");
+
+        nni_mtx_lock(&tq->tq_mtx);
 	for (;;) {
 		if ((task = nni_list_first(&tq->tq_tasks)) != NULL) {
 
-			nni_mtx_lock(&task->task_mtx);
 			nni_list_remove(&tq->tq_tasks, task);
-			nni_mtx_unlock(&task->task_mtx);
 
 			nni_mtx_unlock(&tq->tq_mtx);
 
@@ -183,6 +183,20 @@ nni_task_prep(nni_task *task)
 	nni_mtx_unlock(&task->task_mtx);
 }
 
+void
+nni_task_abort(nni_task *task)
+{
+	// This is called when unscheduling the task.
+	nni_mtx_lock(&task->task_mtx);
+	if (task->task_prep) {
+		task->task_prep = false;
+		task->task_busy--;
+		if (task->task_busy == 0) {
+			nni_cv_wake(&task->task_cv);
+		}
+	}
+	nni_mtx_unlock(&task->task_mtx);
+}
 void
 nni_task_wait(nni_task *task)
 {
