@@ -1,6 +1,6 @@
 //
+// Copyright 2021 Capitar IT Group BV <info@capitar.com>
 // Copyright 2020 Staysail Systems, Inc. <info@staysail.tech>
-// Copyright 2018 Capitar IT Group BV <info@capitar.com>
 //
 // This software is supplied under the terms of the MIT License, a
 // copy of which should be located in the distribution where this
@@ -59,16 +59,16 @@ typedef struct zt_frag     zt_frag;
 typedef struct zt_fraglist zt_fraglist;
 
 // Port numbers are stored as 24-bit values in network byte order.
-#define ZT_GET24(ptr, v)                              \
-	v = (((uint32_t)((uint8_t)(ptr)[0])) << 16) + \
-	    (((uint32_t)((uint8_t)(ptr)[1])) << 8) +  \
-	    (((uint32_t)(uint8_t)(ptr)[2]))
+#define ZT_GET24(ptr, v)                                \
+	v = (((uint32_t) ((uint8_t) (ptr)[0])) << 16) + \
+	    (((uint32_t) ((uint8_t) (ptr)[1])) << 8) +  \
+	    (((uint32_t) (uint8_t) (ptr)[2]))
 
-#define ZT_PUT24(ptr, u)                                     \
-	do {                                                 \
-		(ptr)[0] = (uint8_t)(((uint32_t)(u)) >> 16); \
-		(ptr)[1] = (uint8_t)(((uint32_t)(u)) >> 8);  \
-		(ptr)[2] = (uint8_t)((uint32_t)(u));         \
+#define ZT_PUT24(ptr, u)                                       \
+	do {                                                   \
+		(ptr)[0] = (uint8_t) (((uint32_t) (u)) >> 16); \
+		(ptr)[1] = (uint8_t) (((uint32_t) (u)) >> 8);  \
+		(ptr)[2] = (uint8_t) ((uint32_t) (u));         \
 	} while (0)
 
 static const uint16_t     zt_ethertype = 0x901;
@@ -205,7 +205,7 @@ struct zt_pipe {
 	nni_aio *       zp_ping_aio;
 	uint8_t *       zp_send_buf;
 	nni_atomic_flag zp_reaped;
-	nni_reap_item   zp_reap;
+	nni_reap_node   zp_reap;
 };
 
 typedef struct zt_creq zt_creq;
@@ -454,7 +454,7 @@ zt_node_to_mac(uint64_t node, uint64_t nwid)
 	// multicast and set local administration -- this is the first
 	// octet of the 48 bit mac address.  We also avoid 0x52, which
 	// is known to be used in KVM, libvirt, etc.
-	mac = ((uint8_t)(nwid & 0xfe) | 0x02);
+	mac = ((uint8_t) (nwid & 0xfe) | 0x02);
 	if (mac == 0x52) {
 		mac = 0x32;
 	}
@@ -895,6 +895,7 @@ zt_pipe_recv_data(zt_pipe *p, const uint8_t *data, size_t len)
 		memset(fl->fl_missing, 0xff, nfrags / 8);
 		fl->fl_missing[nfrags / 8] |= ((1 << (nfrags % 8)) - 1);
 	}
+
 	if ((nfrags != fl->fl_nfrags) || (fragsz != fl->fl_fragsz) ||
 	    (fragno >= nfrags) || (fragsz == 0) || (nfrags == 0) ||
 	    ((fragno != (nfrags - 1)) && (len != fragsz))) {
@@ -905,7 +906,7 @@ zt_pipe_recv_data(zt_pipe *p, const uint8_t *data, size_t len)
 		return;
 	}
 
-	bit = (uint8_t)(1 << (fragno % 8));
+	bit = (uint8_t) (1 << (fragno % 8));
 	if ((fl->fl_missing[fragno / 8] & bit) == 0) {
 		// We've already got this fragment, ignore it.  We don't
 		// bother to check for changed data.
@@ -1340,7 +1341,6 @@ zt_wire_packet_send(ZT_Node *node, void *userptr, void *thr, int64_t socket,
 	buf += sizeof(*hdr);
 
 	memcpy(buf, data, len);
-	nni_aio_set_data(aio, 0, hdr);
 	hdr->sa  = addr;
 	hdr->len = len;
 	nni_aio_set_input(aio, 0, &hdr->sa);
@@ -1578,12 +1578,11 @@ done:
 	return (0);
 }
 
-static int
+static void
 zt_tran_init(void)
 {
 	nni_mtx_init(&zt_lk);
 	NNI_LIST_INIT(&zt_nodes, zt_node, zn_link);
-	return (0);
 }
 
 static void
@@ -1713,11 +1712,16 @@ zt_pipe_fini(void *arg)
 	NNI_FREE_STRUCT(p);
 }
 
+static nni_reap_list zt_reap_list = {
+	.rl_offset = offsetof(zt_pipe, zp_reap),
+	.rl_func   = zt_pipe_fini,
+};
+
 static void
 zt_pipe_reap(zt_pipe *p)
 {
 	if (!nni_atomic_flag_test_and_set(&p->zp_reaped)) {
-		nni_reap(&p->zp_reap, zt_pipe_fini, p);
+		nni_reap(&zt_reap_list, p);
 	}
 }
 
@@ -1812,6 +1816,8 @@ zt_pipe_send(void *arg, nni_aio *aio)
 	uint16_t fragno;
 	size_t   fragsz;
 	size_t   bytes;
+	size_t   msg_header_len;
+	size_t   msg_len;
 	nni_msg *m;
 
 	if (nni_aio_begin(aio) != 0) {
@@ -1833,14 +1839,16 @@ zt_pipe_send(void *arg, nni_aio *aio)
 	fragsz = p->zp_mtu - zt_offset_data_data;
 	NNI_ASSERT(fragsz < 0x10000); // Because zp_mtu is 16 bits
 
-	bytes = nni_msg_header_len(m) + nni_msg_len(m);
+	msg_header_len = nni_msg_header_len(m);
+	msg_len        = nni_msg_len(m);
+	bytes          = msg_header_len + msg_len;
 	if (bytes >= (0xfffe * fragsz)) {
 		nni_aio_finish_error(aio, NNG_EMSGSIZE);
 		nni_mtx_unlock(&zt_lk);
 		return;
 	}
 	// above check means nfrags will fit in 16-bits.
-	nfrags = (uint16_t)((bytes + (fragsz - 1)) / fragsz);
+	nfrags = (uint16_t) ((bytes + (fragsz - 1)) / fragsz);
 
 	// get the next message ID, but skip 0
 	if ((id = p->zp_next_msgid++) == 0) {
@@ -1856,29 +1864,27 @@ zt_pipe_send(void *arg, nni_aio *aio)
 		size_t   len;
 
 		// Prepend the header first.
-		if ((len = nni_msg_header_len(m)) > 0) {
-			if (len > fragsz) {
+		if ((!offset) && (msg_header_len > 0)) {
+			if (msg_header_len > fragsz) {
 				// This shouldn't happen!  SP headers are
 				// supposed to be quite small.
 				nni_aio_finish_error(aio, NNG_EMSGSIZE);
 				nni_mtx_unlock(&zt_lk);
 				return;
 			}
-			memcpy(dest, nni_msg_header(m), len);
-			dest += len;
-			room -= len;
-			offset += len;
-			fraglen += len;
-			nni_msg_header_clear(m);
+			memcpy(dest, nni_msg_header(m), msg_header_len);
+			dest += msg_header_len;
+			room -= msg_header_len;
+			offset += msg_header_len;
+			fraglen += msg_header_len;
 		}
 
-		len = nni_msg_len(m);
+		len = msg_header_len + msg_len - offset;
 		if (len > room) {
 			len = room;
 		}
-		memcpy(dest, nni_msg_body(m), len);
+		memcpy(dest, nni_msg_body(m) + offset - msg_header_len, len);
 
-		nng_msg_trim(m, len);
 		NNI_PUT16(data + zt_offset_data_id, id);
 		NNI_PUT16(data + zt_offset_data_fragsz, (uint16_t) fragsz);
 		NNI_PUT16(data + zt_offset_data_frag, fragno);
@@ -1888,7 +1894,7 @@ zt_pipe_send(void *arg, nni_aio *aio)
 		fragno++;
 		zt_send(p->zp_ztn, p->zp_nwid, zt_op_data, p->zp_raddr,
 		    p->zp_laddr, data, fraglen + zt_offset_data_data);
-	} while (nni_msg_len(m) != 0);
+	} while (msg_header_len + msg_len - offset != 0);
 	nni_mtx_unlock(&zt_lk);
 
 	// NB, We never bothered to call nn_aio_sched, because we run this
@@ -2789,7 +2795,7 @@ zt_ep_set_clear_local_addrs(void *arg, const void *data, size_t sz, nni_type t)
 		nni_mtx_unlock(&zt_lk);
 		return (rv);
 	}
-	zn = ep->ze_ztn;
+	zn = ep->ze_ztn->zn_znode;
 	ZT_Node_clearLocalInterfaceAddresses(zn);
 	nni_mtx_unlock(&zt_lk);
 	return (0);
@@ -3225,7 +3231,6 @@ static nni_tran_listener_ops zt_listener_ops = {
 // This is the ZeroTier transport linkage, and should be the
 // only global symbol in this entire file.
 static struct nni_tran zt_tran = {
-	.tran_version  = NNI_TRANSPORT_VERSION,
 	.tran_scheme   = "zt",
 	.tran_dialer   = &zt_dialer_ops,
 	.tran_listener = &zt_listener_ops,
@@ -3234,8 +3239,16 @@ static struct nni_tran zt_tran = {
 	.tran_fini     = zt_tran_fini,
 };
 
+#ifndef NNG_ELIDE_DEPRECATED
 int
 nng_zt_register(void)
 {
-	return (nni_tran_register(&zt_tran));
+	return (nni_init());
+}
+#endif
+
+void
+nni_sp_zt_register(void)
+{
+	nni_tran_register(&zt_tran);
 }
