@@ -205,9 +205,9 @@ extern int nni_http_server_set_tls(nni_http_server *, struct nng_tls_config *);
 extern int nni_http_server_get_tls(
     nni_http_server *, struct nng_tls_config **);
 
-extern int nni_http_server_setx(
+extern int nni_http_server_set(
     nni_http_server *, const char *, const void *, size_t, nni_type);
-extern int nni_http_server_getx(
+extern int nni_http_server_get(
     nni_http_server *, const char *, void *, size_t *, nni_type);
 
 // nni_http_server_start starts listening on the supplied port.
@@ -336,8 +336,7 @@ extern int nni_http_handler_set_method(nni_http_handler *, const char *);
 extern int nni_http_handler_set_data(nni_http_handler *, void *, nni_cb);
 
 // nni_http_handler_get_data returns the data that was previously stored
-// at that index.  It returns NULL if no data was set, or an invalid index
-// is supplied.
+// by nni_http_handler_set_data.
 extern void *nni_http_handler_get_data(nni_http_handler *);
 
 // nni_http_handler_get_uri returns the URI set on the handler.
@@ -360,9 +359,9 @@ extern int nni_http_client_set_tls(nni_http_client *, struct nng_tls_config *);
 extern int nni_http_client_get_tls(
     nni_http_client *, struct nng_tls_config **);
 
-extern int nni_http_client_setx(
-    nni_http_client *, const char *, const void *, size_t, nni_type);
-extern int nni_http_client_getx(
+extern int nni_http_client_set(
+    nni_http_client *, const char *, const void *buf, size_t, nni_type);
+extern int nni_http_client_get(
     nni_http_client *, const char *, void *, size_t *, nni_type);
 
 extern void nni_http_client_connect(nni_http_client *, nni_aio *);
@@ -383,4 +382,8 @@ extern void nni_http_transact_conn(
 extern void nni_http_transact(
     nni_http_client *, nni_http_req *, nni_http_res *, nni_aio *);
 
+// nni_http_stream_scheme returns the underlying stream scheme for a given
+// upper layer scheme.
+extern const char *nni_http_stream_scheme(const char *);
+
 #endif // NNG_SUPPLEMENTAL_HTTP_HTTP_API_H
